import pathlib
import time
import uuid
import os
import csv
import random
import json
from rich.console import Console
from rich.markdown import Markdown

LEVEL = "LEVEL"
QUESTION = "QUESTION"
ANSWER = "ANSWER"
EXPLICATION = "EXPLICATION" 
POSSIBILITIES = "POSSIBILITIES"
UUID = "UUID"


class Question(object):
    """
    class to hold the question data and methods
    """

    def __init__(self, data):
        self._data = data
        self._uuid = uuid.uuid1()
        self._clean_data()

    def _clean_data(self):
        """
        TODO needs quite bit of actual cleanup to make the parsing more robust
        TODO needs a 'private' variable for issue 5
        """
        self._level = self._data[LEVEL].strip()
        self._question = self._data[QUESTION].strip()
        self._answers = self._data[ANSWER].strip().split(" ")
        self._answers = [x for x in self._answers if x]
        self._create_list_of_possibilities()
        try:
            self._explication = self._data[EXPLICATION].strip()
        except:
            self._explication = "No explication available for this question"

    def dump_json(self):
        """
        dumps all data to JSON for the REST API
        TODO needs a key to include the date for issue 5
        """
        data = {
                UUID: self.get_uuid(),
                QUESTION: self.get_question(),
                ANSWER: self.get_right_answers(),
                POSSIBILITIES: self.get_possibilities(),
                EXPLICATION: self.get_explication()
                }
        return json.dumps(data)

    def get_level(self):
        return self._level

    def get_uuid(self):
        return str(self._uuid)

    def get_question(self):
        return self._question

    def get_explication(self):
        """returns the explication as a string"""
        return self._explication

    def _create_list_of_possibilities(self):
        """creates and cleans a list of all the possible answers"""
        possibilities = []
        for key in self._data.keys():
            if key.isnumeric():
                possibilities.append(self._data[key])
        possibilities = [x for x in possibilities if x] # hack to remove empty objects
        self._possibilities = possibilities

    def get_possibilities(self):
        return self._possibilities
        
    def verify(self, answers):
        """needs quite some work"""
        if not isinstance(answers, list):
            raise TypeError
        if len(answers) == 0:
            return False
        right_answers = list(self._answers) # need a copy so we don't change for future questions
        for answer in answers:
            try:
                test = right_answers.index(answer)
                right_answers.pop(test)
            except:
                return False
        if len(right_answers) == 0:
            return True
        else:
            return False

    def get_right_answers(self):
        answers = []
        for answer in self._answers:
            if len(answer) == 0:
                print("question not complete...")
            else:
                answers.append(self._data[answer])
        return answers


class Database(object):
    """holds all the Question objects and methods to get new questions"""

    def __init__(self, filepath, level):
        self.filepath = filepath
        self.level = level
        self._db = []
        self.shuffled_db = []
        self.setup()

    def setup(self):
        if self.filepath.is_dir():
            for filepath in self.filepath.glob("*.csv"):
                data = csv.DictReader(open(filepath, "r"))
                for row in data:
                    try:
                        question = Question(row)
                        if question.get_level() == self.level:
                            self._db.append(question)
                    except Exception as e:
                        print(e)
        else:
            data = csv.DictReader(open(self.filepath, "r"))
            for row in data:
                self._db.append(Question(row))

    def get_question(self):
        if len(self.shuffled_db) == 0:
            self.shuffled_db = list(self._db)
            random.shuffle(self.shuffled_db)
        return self.shuffled_db.pop()

    def get_question_by_uuid(self, uuid):
        question = [question for question in self._db if question.get_uuid() == uuid]
        if len(question) == 1:
            return question[0]
        elif len(question) > 1:
            raise Exception
        else:
            return None


class Game(object):
    def __init__(self, filepath):
        self._filepath = pathlib.Path(filepath)
        self._players = []
        self.setup_players()
        self.shuffled_list = []

    def setup_players(self):
        with open(self._filepath, "r") as fp:
            lines = fp.readlines()
            for line in lines:
                self.add_player(line.strip()) 

    def get_random_player(self):
        if len(self.shuffled_list) == 0:
            self.shuffled_list = list(self._players)
            random.shuffle(self.shuffled_list)
        return self.shuffled_list.pop()

    def get_all_players(self):
        return self._players

    def add_player(self, name):
        player = Player(name)
        self._players.append(player)


class Stats(object):
    """TODO placeholder for the stats class"""
    def __init__(self):
        self._stats = []

    def update_stats(self, stat):
        self._stats.append(stat)

    def get_stats(self):
        return self.get_right(), self.get_wrong(), len(self._stats)

    def get_wrong(self):
        return self._stats.count(False)

    def get_right(self):
        return self._stats.count(True)


class Player(object):
    def __init__(self, name):
        self.name = name
        self.stats = Stats()

    def get_stats(self):
        return self.name, self.stats.get_right(), self.stats.get_wrong()


if __name__ == "__main__":
    filepath = pathlib.Path("../data/list_book1.csv")
    db = Database(filepath)
    for i in range(0, 10):
        q = db.get_question()
        uid = q.get_uuid()
        print(uid)
        t = db.get_question_by_uuid(uid)
        print(t.get_uuid())

